// SPDX-License-Identifier: MIT OR Apache-2.0

#ifndef LEWTON_LEWTON_H
#define LEWTON_LEWTON_H


#define LEWTON_MAJOR 0
#define LEWTON_MINOR 10
#define LEWTON_PATCH 2


#include <stddef.h>
#include <stdint.h>
#include <stdlib.h>

/**
 * Main Decoder State
 *
 * It is created by `lewton_context_from_extradata` by passing a xiph-laced extradate bundle
 */
typedef struct LewtonContext LewtonContext;

/**
 * A multichannel vector of samples
 *
 * It is produced by `lewton_decode_packet`
 *
 * Use `lewton_samples_count` to retrieve the number of samples available in each channel
 * Use `lewton_samples_channels` to retrieve the number of channels
 * Use `lewton_samples_for_channel_f32` to retrieve a reference to the data present in the
 * channel
 *
 * use `lewton_samples_drop()` to deallocate the memory
 */
typedef struct LewtonSamples LewtonSamples;

/**
 * Create a LewtonContext from an extradata buffer
 *
 * Returns either NULL or a newly allocated LewtonContext
 */
LewtonContext *lewton_context_from_extradata(const uint8_t *data, uintptr_t len);

/**
 * Reset the Decoder to support seeking.
 */
void lewton_context_reset(LewtonContext *ctx);

/**
 * Decode a packet to LewtonSamples when possible
 *
 * Returns 0 on success, non-zero if no samples can be produced
 */
int lewton_decode_packet(LewtonContext *ctx,
                         const uint8_t *pkt,
                         uintptr_t len,
                         LewtonSamples **sample_out);

/**
 * Provide the number of samples present in each channel
 */
uintptr_t lewton_samples_count(const LewtonSamples *samples);

void lewton_samples_drop(LewtonSamples *samples);

void lewton_context_drop(LewtonContext *ctx);

#endif  /* LEWTON_LEWTON_H */
